

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%% driverVSBBO.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file illustrates the use of VSBBO
% by showing how to minimize the function f(x):=||Ax-b||_p^e
% for p=2,e=1 in dimension n=300 from a random starting point.
% The goal assumed in the example is the reduction of the initial 
% objective function value by a factor of 0.01 with at most nfmax=200*n 
% function evaluations, to return when one of the two limiting 
% conditions is first satisfied.

clear;

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% create problem definition

% define problem parameters (to be adapted to your problem)
n=3;          % dimension
p=2;          % norm in objective function
e=1;          % exponent in objective function 

% create random matrix and right hand side for objective function
% (specific to the model problem; replace by whatever data you
% need to provide to your objectiv function)
A=rand(n)-0.5; 
x0=rand(n,1)-0.5;
x=A*x0;
b=-sum(A,2);

% create objective function f(x)=||Ax-b||_p^e
fun=@(x) norm(A*x-b,p).^e; 
% To solve your own problem, simply replace in the previous line 
% the expression after @(x) by your expression or function call. 
% Parameters in this expression are passed by value, hence are 
% fixed during minimization.

% start and stop info
x=2*rand(n,1)-1;      % starting point
nfmax=200*n;          % stop after nfmax function evaluations
secmax=inf;           % stop after secmax seconds
ftarget=1e-8*fun(x);  % stop when a point with f(x)<=ftarget is found
                      % (set ftarget=-inf if no target is known)

% customization
prt=-1;          % print level 
                %   -1: nothing, 0: litte, >=1: more and more
noTuning=1;     % no tuning of VSBBO? (recommended at first)
fullTuning=0;   % full tuning? (for specialists only)

% problem definition complete
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% solve the problem with VSSBO

tic; % set clock

% pass stop and print criteria
% (indefinite run if no stopping criterion is given)
st.secmax=secmax;       % (default: inf)
st.nfmax=nfmax;         % (default: inf)
st.ftarget=ftarget;     % (default: -inf)
st.prt=prt;             % (default: -1)

if noTuning, 
  % call VSBBO 
  [x,f,info] = VSBBO(fun,x,st);
end;

if fullTuning,
  % set tuning parameters (for specialists only)
  % given are defaults; these wouldn't need to be set 
  
  % maximal number of best points kept
  tune.mmax = 5; 
  % maximal number of multi-line searches in setScales
  tune.T0 = 50*n;
  % maximal number of coordinate trials
  tune.C = n;
  % maximal number of subspace directions in multi-line search
  tune.S = min(fix(n/10)+1,5); 
  % maximal number of random trials 
  tune.R = min(fix(n/10)+1,20);
  % maximal number of extrapolations in MLS
  tune.E= Inf; 
  % scale for subspace direction
  tune.scSub = 0; 
  % scale for cumulative direction
  tune.scCum = 0; 
  % select cumulative step type
  tune.cum = 1; 
  if tune.cum==2
    % upper bound for estimating cumulative step-size
    tune.a = 1;
  end
  % minimium threshold for good impovement
  tune.Deltamin = 0;
  % initial threshold for good impovement
  tune.Deltamax = 1e-6;
  % factor for finding delta
  tune.gammadelta = 1e+6; 
  % factor for adjusting Deltamax
  tune.gammamax = 1e-6; 
  % factor for extrapolation test
  tune.gammaE = 4;
  % factor for finding initial lam
  tune.gammalambda = 1e-6; 
  % factor for adjusting Delta
  tune.Q = 2;
  % factor for angle condition
  tune.gammaangle = 1e-20;
  % factor for angle condition
  tune.gammaw = eps;
  % call VSBBO 
  [x,f,info] = VSBBO(fun,x,st,tune);
end
      
      
% display output

if prt<0, disp('VSBBO completed silently'); end;
disp(' ');
disp('display output');
disp(' ');
info               % progress report by VSBBO
nstep=info.nstep
nfused=info.nf                  % number of function evaluations used
secused=cputime-info.initTime   % time used up to now
x                               % best point found     
f                               % function value at xbest    
ftarget                         % target (for comparison)
